@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2008 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2009 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Portions Copyright (c) 1997 Alan Citterman. All rights reserved.
 #
@@ -30,7 +30,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.58";
+$VERSION = "0.65";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -81,19 +81,31 @@ my $last_new_err = Text::CSV_XS->SetDiag (0);
 
 sub new
 {
-    $last_new_err =
-	"usage: my \$csv = Text::CSV_XS->new ([{ option => value, ... }]);";
+    $last_new_err = SetDiag (undef, 1000,
+	"usage: my \$csv = Text::CSV_XS->new ([{ option => value, ... }]);");
+
     my $proto = shift;
     my $attr  = shift || {};
     my $class = ref ($proto) || $proto	or return;
     for (keys %{$attr}) {
-	m/^[a-z]/ && exists $def_attr{$_} and next;
+	if (m/^[a-z]/ && exists $def_attr{$_}) {
+	    $] >= 5.008002 && m/_char$/ and utf8::decode ($attr->{$_});
+	    next;
+	    }
 #	croak?
-	$last_new_err = "Unknown attribute '$_'";
+	$last_new_err = SetDiag (undef, 1000, "INI - Unknown attribute '$_'");
 	return;
 	}
-    $last_new_err = SetDiag (undef, 0);
+
     my $self  = {%def_attr, %{$attr}};
+    if ($self->{allow_whitespace} and
+	    (defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/) ||
+	    (defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/)) {
+	$last_new_err = SetDiag (undef, 1002);
+	return;
+	}
+
+    $last_new_err = SetDiag (undef, 0);
     defined $\ && !exists $attr->{eol} and $self->{eol} = $\;
     bless $self, $class;
     defined $self->{types} and $self->types ($self->{types});
@@ -126,6 +138,7 @@ sub _set_attr_C
 {
     my ($self, $name, $val) = @_;
     defined $val or $val = 0;
+    $] >= 5.008002 and utf8::decode ($val);
     $self->{$name} = $val;
     $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
@@ -137,6 +150,7 @@ sub _set_attr_N
 {
     my ($self, $name, $val) = @_;
     $self->{$name} = $val;
+    $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
     my $i = $_cache_id{$name};
     $cache[$i++] = $_ for unpack "C*", pack "N", $val;
@@ -148,14 +162,24 @@ sub _set_attr_N
 sub quote_char
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("quote_char", shift);
+    if (@_) {
+	my $qc = shift;
+	defined $qc && $qc =~ m/^[ \t]$/ && $self->{allow_whitespace} and
+	    croak ($self->SetDiag (1002));
+	$self->_set_attr_C ("quote_char", $qc);
+	}
     $self->{quote_char};
     } # quote_char
 
 sub escape_char
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("escape_char", shift);
+    if (@_) {
+	my $ec = shift;
+	defined $ec && $ec =~ m/^[ \t]$/ && $self->{allow_whitespace} and
+	    croak ($self->SetDiag (1002));
+	$self->_set_attr_C ("escape_char", $ec);
+	}
     $self->{escape_char};
     } # escape_char
 
@@ -227,7 +251,14 @@ sub allow_loose_escapes
 sub allow_whitespace
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("allow_whitespace", shift);
+    if (@_) {
+	my $aw = shift;
+	$aw and
+	  (defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/) ||
+	  (defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/) and
+	    croak ($self->SetDiag (1002));
+	$self->_set_attr_C ("allow_whitespace", $aw);
+	}
     $self->{allow_whitespace};
     } # allow_whitespace
 
@@ -281,15 +312,14 @@ sub error_input
 sub error_diag
 {
     my $self = shift;
-    my @diag = (0, $last_new_err, 0);
+    my @diag = (0 + $last_new_err, $last_new_err, 0);
 
-    unless ($self && ref $self) {	# Class method or direct call
-	$last_new_err and $diag[0] = 1000;
-	}
-    elsif ($self->isa (__PACKAGE__) && exists $self->{_ERROR_DIAG}) {
+    if ($self && ref $self && # Not a class method or direct call
+	 $self->isa (__PACKAGE__) && exists $self->{_ERROR_DIAG}) {
 	@diag = (0 + $self->{_ERROR_DIAG}, $self->{_ERROR_DIAG});
 	exists $self->{_ERROR_POS} and $diag[2] = 1 + $self->{_ERROR_POS};
 	}
+
     my $context = wantarray;
     unless (defined $context) {	# Void context
 	$diag[0] and print STDERR "# CSV_XS ERROR: $diag[0] - $diag[1]\n";
@@ -650,6 +680,10 @@ FOR NEWLINE), U+FF02 (FULLWIDTH QUOTATION MARK), and U+201C (LEFT DOUBLE
 QUOTATION MARK) (to give some examples of what might look promising) are
 therefor not allowed.
 
+If you use perl-5.8.2 or higher, these three attributes are utf8-decoded, to
+increase the likelyhood of success. This way U+00FE will be allowed as a
+quote character.
+
 =item *
 
 A field within CSV must be surrounded by double-quotes to contain an embedded
@@ -704,12 +738,16 @@ See also CAVEATS
 =item allow_whitespace
 
 When this option is set to true, whitespace (TAB's and SPACE's)
-surrounding the separation character is removed when parsing. So
-lines like:
+surrounding the separation character is removed when parsing. If
+either TAB or SPACE is one of the three major characters C<sep_char>,
+C<quote_char>, or C<escape_char> it will not be considered whitespace.
+
+So lines like:
 
   1 , "foo" , bar , 3 , zapp
 
 are now correctly parsed, even though it violates the CSV specs.
+
 Note that B<all> whitespace is stripped from start and end of each
 field. That would make is more a I<feature> than a way to be able
 to parse bad CSV lines, as
@@ -912,11 +950,11 @@ If the C<new ()> constructor call fails, it returns C<undef>, and makes
 the fail reason available through the C<error_diag ()> method.
 
  $csv = Text::CSV_XS->new ({ ecs_char => 1 }) or
-     die Text::CSV_XS->error_diag ();
+     die "".Text::CSV_XS->error_diag ();
 
 C<error_diag ()> will return a string like
 
- "Unknown attribute 'ecs_char'"
+ "INI - Unknown attribute 'ecs_char'"
 
 =head2 combine
 
@@ -936,15 +974,16 @@ invalid argument.
 Similar to combine, but it expects an array ref as input (not an array!)
 and the resulting string is not really created, but immediately written
 to the I<$io> object, typically an IO handle or any other object that
-offers a I<print> method. Note, this implies that the following is wrong:
+offers a I<print> method. Note, this implies that the following is wrong
+in perl 5.005_xx and older:
 
  open FILE, ">", "whatever";
  $status = $csv->print (\*FILE, $colref);
 
-The glob C<\*FILE> is not an object, thus it doesn't have a print
-method. The solution is to use an IO::File object or to hide the
-glob behind an IO::Wrap object. See L<IO::File(3)> and L<IO::Wrap(3)>
-for details.
+as in perl 5.005 and older, the glob C<\*FILE> is not an object, thus it
+doesn't have a print method. The solution is to use an IO::File object or
+to hide the glob behind an IO::Wrap object. See L<IO::File(3)> and
+L<IO::Wrap(3)> for details.
 
 For performance reasons the print method doesn't create a result string.
 In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
@@ -1029,7 +1068,7 @@ fetched fields in, C<getline ()> will fail. If you pass more than there are
 fields to return, the remaining references are left untouched.
 
   $csv->bind_columns (\$code, \$name, \$price, \$description);
-  while ($csv->getline ()) {
+  while ($csv->getline ($io)) {
       print "The price of a $name is \x{20ac} $price\n";
       }
 
@@ -1087,7 +1126,11 @@ Set field type to string.
  @columns = $csv->fields ();
 
 This object function returns the input to C<combine ()> or the resultant
-decomposed fields of C<parse ()>, whichever was called more recently.
+decomposed fields of C successfull <parse ()>, whichever was called more
+recently.
+
+Note that the return value is undefined after using C<getline ()>, which
+does not fill the data structures returned by C<parse ()>.
 
 =head2 meta_info
 
@@ -1263,10 +1306,33 @@ Reading a CSV file line by line:
   close $fh or die "file.csv: $!";
 
 For more extended examples, see the C<examples/> subdirectory in the
-original distribution. Included is C<examples/parser-xs.pl>, that could
-be used to `fix' bad CSV and parse beyond errors.
+original distribution. The following files can be found there:
 
-  perl examples/parser-xs.pl bad.csv >good.csv
+=over 2
+
+=item parser-xs.pl
+
+This can be used as a boilerplate to `fix' bad CSV and parse beyond errors.
+
+  $ perl examples/parser-xs.pl bad.csv >good.csv
+
+=item csv-check
+
+This is a command-line tool that uses parser-xs.pl techniques to check the
+CSV file and report on its content.
+
+  $ csv-check files/utf8.csv
+  Checked with examples/csv-check 1.2 using Text::CSV_XS 0.61
+  OK: rows: 1, columns: 2
+      sep = <,>, quo = <">, bin = <1>
+
+=item csv2xls
+
+A script to convert CSV to Microsoft Excel. This requires L<Date::Calc(3)>
+and L<Spreadsheet::WriteExcel(3)>. The converter acceps various options and
+can produce UTF-8 Excel files.
+
+=back
 
 =head1 CAVEATS
 
@@ -1289,13 +1355,6 @@ setting in it, so checking the locale is no solution.
 
 =over 2
 
-=item More tests
-
-For all possible errors, there should be a test.
-
-All XS code should be covered in the test cases, except for perl
-internal failure, like failing to store a hash value.
-
 =item More Errors & Warnings
 
 New extensions ought to be clear and concise in reporting what error
@@ -1322,11 +1381,6 @@ C<string ()> in a single call will B<not> be honored. Likewise for
 C<parse ()> and C<fields ()>. Given the trouble with embedded newlines,
 Using C<getline ()> and C<print ()> instead is the prefered way to go.
 
-=item Unicode
-
-We probably need many more tests to check if all edge-cases are covered.
-See t/50_utf8.t.
-
 =item Parse the whole file at once
 
 Implement a new methods that enables the parsing of a complete file
@@ -1383,6 +1437,9 @@ on the cause of the failure. Note that for speed reasons, the internal value
 is never cleared on success, so using the value returned by C<error_diag ()> in
 normal cases - when no error occured - may cause unexpected results.
 
+If the constructor failed, the cause can be found using C<error_diag ()> as a
+class method, like C<Text::CSV_XS->error_diag ()>.
+
 Currently errors as described below are available. I've tried to make the error
 itself explanatory enough, but more descriptions will be added. For most of
 these errors, the first three capitals describe the error category:
@@ -1426,6 +1483,11 @@ HashRef parse related error.
 The separation character cannot be equal to either the quotation character
 or the escape character, as that will invalidate all parsing rules.
 
+=item 1002 "INI - allow_whitespace with escape_char or quote_char SP or TAB"
+
+Using C<allow_whitespace> when either C<escape_char> or C<quote_char> is
+equal to SPACE or TAB is too ambiguous to allow.
+
 =item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 
 When C<eol> has been set to something specific, other than the default,
@@ -1520,7 +1582,7 @@ exhausted before the quote is found, that field is not terminated.
 =head1 SEE ALSO
 
 L<perl(1)>, L<IO::File(3)>, L<IO::Handle(3)>, L<IO::Wrap(3)>,
-L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Encoded>,
+L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Encoded(3)>,
 L<Text::CSV::Separator(3)>, and L<Spreadsheet::Read(3)>.
 
 =head1 AUTHORS and MAINTAINERS
@@ -1538,12 +1600,12 @@ and the print and getline methods. See ChangeLog releases 0.10 through
 
 H.Merijn Brand F<E<lt>h.m.brand@xs4all.nlE<gt>> cleaned up the code,
 added the field flags methods, wrote the major part of the test suite,
-completed the documentation, fixed some RT bugs. See ChangeLog releases
-0.25 and on.
+completed the documentation, fixed some RT bugs and added all the allow
+flags. See ChangeLog releases 0.25 and on.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2007-2008 H.Merijn Brand for PROCURA B.V.
+Copyright (C) 2007-2009 H.Merijn Brand for PROCURA B.V.
 Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
 Portions Copyright (C) 1997 Alan Citterman. All rights reserved.
 
